/*
 * Copyright (C) 2002, 2003, 2004 Emmanuel Saracco <esaracco@noos.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef H_UC_CONNECTION
#define H_UC_CONNECTION

#include "general.h"

/**
 * UCTcpState:
 * @UC_TCP_STATE_CONNECTING: We are not yes connected.
 * @UC_TCP_STATE_CONNECTED: We are connected.
 * @UC_TCP_STATE_CLOSED: Connection have been closed.
 *
 * The different states of a #UCConn object.
 *
 */
typedef enum _UCTcpState UCTcpState;
enum _UCTcpState
{
  UC_TCP_STATE_CONNECTING,
  UC_TCP_STATE_CONNECTED,
  UC_TCP_STATE_CLOSED
};

/**
 * UCHTTPStatus:
 * @UC_HTTP_STATUS_UNKNOWN: Not defined.
 * @UC_HTTP_STATUS_OK: Host was conntacted without problem.
 * @UC_TCP_STATUS_TIMEOUT: A timeout occured while connecting.
 * @UC_HTTP_STATUS_BAD_HOSTNAME: Bad hostname (can not resolve).
 * @UC_HTTP_STATUS_CONNECTION_REFUSED: Connection refused by remote.
 * @UC_TCP_STATUS_ERROR: A error occured.
 *
 * The different status of a HTTP link
 * 
 */
typedef enum _UCTcpStatus UCTcpStatus;
enum _UCTcpStatus
{
  UC_TCP_STATUS_UNKNOWN,
  UC_TCP_STATUS_OK,
  UC_TCP_STATUS_TIMEOUT,
  UC_TCP_STATUS_BAD_HOSTNAME,
  UC_TCP_STATUS_CONNECTION_REFUSED,
  UC_TCP_STATUS_ERROR
};

/**
 * UCConn:
 * @socket: The socket descriptor.
 * @thread_id: current thread.
 * @thread_end: %TRUE when the main thread is not yet running.
 * @thread_ftp_end: %TRUE when the FTP thread is not yet running.
 * @timeout: %TRUE if a timeout occured while retreiving/sending data.
 * @tcp_state: A #UCTcpState value to describe the connection state.
 * @tcp_status: A #UCTcpState value to describe the connection status.
 * @hostname: Name of the host.
 * @port: Port of the host.
 * @array: This array is used for saving server response.
 * @request: The request to send to the server.
 * @session_gnutls: GNU TLS session.
 * @xcred_gnutls: certificate credential.
 * @use_gnutls: %TRUE if the protocol require GNU TLS (HTTPS).
 * @use_ftp: %TRUE if the protocol is FTP.
 * @ftp_passive: %TRUE if we are reading a FTP response.
 * @ftp_passive_socket: Socket to use if we are managing FTP response.
 * @ftp_passive_port: Port to use to read FTP response.
 * 
 * Here we store connection informations.
 */
typedef struct _UCConn UCConn;
struct _UCConn
{
  gint socket;
  pthread_t thread_id;
  gboolean thread_end;
  gboolean thread_ftp_end;
  gboolean timeout;
  UCTcpState tcp_state;
  UCTcpStatus tcp_status;
  gchar *hostname;
  guint port;
  GArray *array;
  gchar *request;
#ifdef ENABLE_GNUTLS
  gnutls_session session_gnutls;
  gboolean use_gnutls;
#endif
  gboolean use_ftp;
  gboolean ftp_passive;
  gint ftp_passive_socket;
  guint ftp_passive_port;
};

void uc_conn_init (void);
GArray *uc_server_get_response (const gchar * proto, gchar * host,
				const guint port, gchar * buffer);

#endif
